/*
 * AbstractCell.java
 * Tomokazu ARITA
 * arita@obirin.ac.jp
 * Created on 2003/08/14, 3:23
 * Update on 2005/02/11
 */


package org.hichart.h3code;
 

/**
 * The <code> AbstractCell </code> class has the common methods 
 * for the <code>Cell</code> and <code>PerimeterCell</code> classes. 
 * @version 0.1.0 2005/02/11
 * @author Tomokazu ARITA (arita@obirin.ac.jp)
 * 
 */

public abstract class AbstractCell extends AbstractLinkedNode{
    
    /* XCb`(position) */
	
    /**
     * The <code>NORTH</code> denotes the north wall position. 
     */
    public static final int NORTH = 1;
    
    /**
     * The <code>SOUTH</code> denotes the south wall position. 
     */
    public static final int SOUTH = 2; 
    
    /**
     * The <code>WEST</code> denotes the west wall position. 
     */
    public static final int WEST = 3;
    
    /**
     * The <code>EAST</code> denotes the east wall position. 
     */
    public static final int EAST = 4;
    
    
    /* XCb`2 */
    
    /**
     * The <code>PERIMETER_CELL</code> is the ID for the perimeter cells. 
     */
    public static final int PERIMETER_CELL = 0;
    
    /**
     * The <code>CELL</code> is the ID for the cells without perimeter cells. 
     */
    public static final int CELL = 1;
    
    /* tB[h */
    /* 2005/02/11 */
    private WallValue northWall = null;
    private WallValue southWall = null;
    private WallValue westWall  = null;
    private WallValue eastWall  = null;
    
    /** Creates a new instance of AbstractCell */
    public AbstractCell(int id){
        super(id);
    }
    
    /* setAllWall */
    /* 4̕ǂxɐݒ */
    /* ݒłꍇtruełȂȂfalse */
    /**
     * Sets all wall value. 
     * @param northWall_value   a value of the north wall   
     * @param southWal_value    a value of the south wall
     * @param westWall_value    a value of the west wall
     * @param eastWall_value    a value of the east wall
     * @return if all value is setted, then <code>setAllWall</code> returns <code>true</code>, else <code>setAllWall</code> returns <code>false</code>.
     */
    public boolean setAllWall(int northWall_value, int southWal_value,  int westWall_value, int eastWall_value){
        if(setWall(northWall_value, NORTH)==false){return false;}
        else if(setWall(southWal_value, SOUTH)==false){return false;}
        else if(setWall(westWall_value, WEST) == false){return false;}
        else if(setWall(eastWall_value, EAST)==false){return false;}
        return true;
    }
    
    /* setWall */
    /* 1̕ǂݒ */
    /* ݒłtrue@oȂfalse */
    /**
     * Sets a wall value. 
     * @param value     the wall value
     * @param position  <code>NORTH</code>, <code>WEST</code>, <code>SOUTH</code>, or <code>EAST</code>.
     * @return if a value is setted, <code>setWall</code> returns <code>true</code>.
     */
    public boolean setWall(int value, int position){
        if(value < 0){ return false;} /* lȂfalse */
        
        switch(position){
            case NORTH:
            	if(northWall != null){ northWall.set(value); return true; }
            	else{ return false; }
            case SOUTH:
            	if(southWall != null){ southWall.set(value); return true; }
            	else{ return false; }
            case WEST:
            	if(westWall != null){ westWall.set(value); return true; }
            	else{ return false; }
            case EAST:
            	if(eastWall != null){ eastWall.set(value); return true; }
            	else{ return false; }
            default:
                return false;
        }
    }
    
    
    /* getWall */
    /* ǂ̒lԂ */
    /* positionԈĂꍇ-1Ԃ */
    
    /**
     * Gets a wall value.
     * @param position   <code>NORTH</code>, <code>WEST</code>, <code>SOUTH</code>, or <code>EAST</code>.
     * @return  a wall value. If the position is wrong, then <code>getWall</code> returns -1.  
     */
    public int getWall(int position){
         switch(position){
            case NORTH:
            	System.out.println("North Wall :" + northWall.get()); //debug
                return northWall.get();
            case SOUTH:
                return southWall.get();
            case WEST:
                return westWall.get();
            case EAST:
                return eastWall.get();
            default:
                return -1;
        }
    }
    
    /* setWallValue */
    /* ǂ̒lǗWallValueNXZbgD */
    /**
     * Sets an instance of <code>WallValue</code>. 
     * @param value     an instance of <code>WallValue</code>. 
     * @param position  <code>NORTH</code>, <code>WEST</code>, <code>SOUTH</code>, or <code>EAST</code>.
     * @return  if the <code>value</code> is setted correctly, then <code>setWallValue</code> returns true. 
     */
    public boolean setWallValue(WallValue value, int position){
    	System.out.println("Node ID (set wall value): " + this.getID()); //Debug code.
    	
    	if(value == null){ return false;} /* lȂfalse */
    	
    	switch(position){
        	case NORTH:
        		northWall = value;
        		return true;
        	case SOUTH:
        		southWall = value; 
        		return true;
        	case WEST:
        		westWall = value; 
        		return true;
        	case EAST:
        		eastWall = value; 
        		return true;
        	default:
        		return false;
    	}
    }
    
    
    /* getWallValue */
    /* ǂ̒lǗWallValueNX̃CX^X𓾂 */
    /**
     * Gets an instance of <code>WallValue</code>. 
     * @param position  <code>NORTH</code>, <code>WEST</code>, <code>SOUTH</code>, or <code>EAST</code>. 
     * @return  an instance of <code>WallValue</code> that is setted on a wall. 
     */
    public WallValue getWallValue(int position){
    	switch(position){
        	case NORTH:
        		return northWall;
        	case SOUTH:
        		return southWall;
        	case WEST:
        		return westWall;
        	case EAST:
        		return eastWall;
        	default:
        		return null;
    	}
    }
    
    /* setAllWallValue */
    /* 4̕ǂxɐݒ(ӂ̐ڑςƂɂ͕KĂяo) */
    /* ݒłꍇtruełȂȂfalse */
    /**
     * Sets all <code>WallValue</code>. 
     * This methods is always called when edges are reconnected. 
     * @param northWall_value   an instance of <code>WallValue</code> for the north wall. 
     * @param southWal_value    an instance of <code>WallValue</code> for the south wall.
     * @param westWall_value    an instance of <code>WallValue</code> for the west wall.
     * @param eastWall_value    an instance of <code>WallValue</code> for the east wall.
     * @return if all value is setted correctly, this method returns true. 
     */
    public boolean setAllWallValue(WallValue northWall_value, WallValue southWal_value,  WallValue westWall_value, WallValue eastWall_value){
        if(setWallValue(northWall_value, NORTH)==false){return false;}
        else if(setWallValue(southWal_value, SOUTH)==false){return false;}
        else if(setWallValue(westWall_value, WEST) == false){return false;}
        else if(setWallValue(eastWall_value, EAST)==false){return false;}
        return true;
    }
    
}
