/*
 * Cell.java
 * Tomokazu ARITA
 * arita@obirin.ac.jp
 * Created on 2003/08/14, 3:59
 */

package org.hichart.h3code;

/**
 * @author Tomokazu ARITA (arita@obirin.ac.jp)
 * @version 0.1.0 2003/08/14
 * 
 */
public class Cell extends AbstractCell{
    
    /**
     * <code>ALIGN_LEFT</code> is an alignment key for aligning contents to left in cell.
     */
    public static final int ALIGN_LEFT = 1;
    
    /**
     * <code>ALIGN_CENTER</code> is an alignment key for aligning contents to center in cell.
     */
    public static final int ALIGN_CENTER = 2;
    
    /**
     * <code>ALIGN_RIGHT</code> is an alignment key for aligning contents to right in cell.
     */
    public static final int ALIGN_RIGHT = 3;
    
    
    /**
     * <code>content</code> is a content instance. 
     */
    private Content content = null; 
    
    
    /**
     * <code>align</code> is the alignment of this instance. 
     */
    private int align = ALIGN_LEFT;
    
    /** Creates a new instance of Cell 
     * @param id  ID number 
     */
    public Cell(int id){
        super(id);
    }
    
    
    /* RecZbg */
    /**
     * Sets content of this cell. 
     * @param content   a content instance. 
     */
    public void setContent(Content content){
        this.content = content;
    }
    
    /* RecԂ */
    /**
     * Gets a content of this cell.
     * @return  a content instance. 
     */
    public Content getContent(){
        return content;
    }
    
    /* Ń\bh */
    /* (non-Javadoc)
     * @see org.hichart.h3code.AbstractLinkedNode#destroy()
     */
    public boolean destroy(){
        content = null;
        return true;
    }
    
    /**
     * Sets an alignment of the content. 
     * @param align   <code>ALIGN_LEFT</code>, <code>ALIGN_CENTER</code>, or <code>ALIGN_RIGHT</code>
     */
    public void setContentAlign(int align){
        this.align = align;
    }
    
    /**
     * Gets the alignment of this instance. 
     * @return  <code>ALIGN_LEFT</code>, <code>ALIGN_CENTER</code>, or <code>ALIGN_RIGHT</code>
     */
    public int getContentAlign(){
        return this.align;
    }
    
}
