/*
 * H3CParser.java
 * Tomokazu ARITA
 * arita@obirin.ac.jp
 * Created on 2003/08/14, 10:57
 * Update on 2005/02/10
 * Update on 2005/12/05 bugfix 
 * update on 2006/01/16 (EWt)
 */

package org.hichart.h3code.tool;

import java.util.StringTokenizer;
//import java.util.TreeSet;
import java.io.BufferedReader;
import java.util.ArrayList;

import org.hichart.h3code.AbstractCell;
import org.hichart.h3code.AbstractLinkedNode;
import org.hichart.h3code.Cell;
//import org.hichart.h3code.PerimeterCell;
//import org.hichart.h3code.Table;
//import org.hichart.h3code.WallValue;

/**
 * H3-Code Parser: creates tables from BufferedReader Stream.  
 * 
 * @author Tomokazu ARITA (arita@obirin.ac.jp)
 * old_version 0.1.0 2003/08/14, 10:57
 * @version 0.1.1 2006/01/16 
 * 
 * 
 */
public class H3CParser {
    
    /* wbhpNX */
    /**
     * An Inner Class of H3CParser for processing the header part in a H3-code file. 
     * @author Tomokazu ARITA (arita@obirin.ac.jp)
     * @version 0.1.0 
     */
    public static class Header{
        
    	/**
    	 * <code>version</code>: h3-code verion
    	 */
    	public int version;
    	
        /**
         * <code>row</code>: the number of rows
         */
        public int row;
        
        
        /**
         * <code>column</code>: the number of columns
         */
        public int column;
    }
    
    
    /* ݏ̃Zf[^p̓NX */
    /**
     * An Inner Class of H3CPerser for processing lines of cells in a H3-Code file.
     * @author Tomokazu ARITA (arita@obirin.ac.jp)
     * @version 0.1.0 
     */
    public static class CellLine{
        
    	/**
    	 * <code>nodeId</code>: ID number
    	 */
    	public int nodeId;
        
    	/**
    	 * <code>type</code>: the type of a cell
    	 */
    	public int type;
    	
        /**
         * <code>nweLeft</code>: pointer -- north wall edge (left)
         */
        public int nweLeft;
        
        /**
         * <code>sweLeft</code>: pointer -- south wall edge (left)
         */
        public int sweLeft;
        
        /**
         * <code>nweRight</code>: pointer -- north wall edge (right) 
         */
        public int nweRight;
        
        
        /**
         * <code>sweRight</code>: pointer -- south wall edge (right)
         */
        public int sweRight;
        
        
        /**
         * <code>wweUpper</code>: pointer -- west wall edge (upper)
         */
        public int wweUpper;
        
        
        /**
         * <code>eweUpper</code>: pointer -- east wall edge (upper)
         */
        public int eweUpper;
        
        
        /**
         * <code>wweLower</code>: pointer -- west wall edge (lower) 
         */
        public int wweLower;
        
        
        /**
         * <code>eweLower</code>: pointer -- east wall edge (lower) 
         */
        public int eweLower;
        
        
        /**
         * <code>northWall</code>: a value of north wall 
         */
        public int northWall;
        
        /**
         * <code>southWall</code>: a value of south wall 
         */
        public int southWall;
        
        /**
         * <code>westWall</code>: a value of west wall 
         */
        public int westWall;
        
        
        /**
         * <code>eastWall</code>: a value of east wall 
         */
        public int eastWall;
        
        /**
         * <code>contentId</code>: content ID number
         */
        public int contentId;
        
        
        /**
         *  <code>contentAlign</code>: the content alignment
         */
        public int contentAlign;
        //public int ePoint; //KvȂ̂Ŗ
    }
    
    
//    /* ZIuWFNgrpRp[^- */
//    public static class CellComparator implements java.util.Comparator{
//        public int compare(Object obj1, Object obj2) {
//            if(obj1 instanceof h3code.AbstractCell && obj2 instanceof h3code.AbstractCell){
//                h3code.AbstractCell c1 = (h3code.AbstractCell) obj1;
//                h3code.AbstractCell c2 = (h3code.AbstractCell) obj2;
//                return c1.getID() -c2.getID();
//            }else{
//                /* IuWFNgɑ΂镶̈ꕶڂr */
//                /* StringComparable */
//               String s1 = obj1.toString().substring(0,1);
//               String s2 = obj2.toString().substring(0,1);
//               return s1.compareTo(s2);
//            }
//        }
//    }
    
//    /*ZW*/
//    private TreeSet cellSet = new TreeSet(new CellComparator());
    
    private Header header = null;
    private CellLine cellLine = null;
    
    /** Creates a new instance of H3CParser */
    public H3CParser(){}
    
    //
    /**
     * Initializes thie parser. 
     */
    public void init(){
//        cellSet.clear();
        header = null;
        cellLine = null;
    }
    
    //
    /**
     * Analyzes H3-code data from a BufferedReader steam and create a table. 
     * @param br     input string steam
     * @param table  a created table
     */
    public void analyze(BufferedReader br, org.hichart.h3code.Table table){
        init();
        try{
            analyzeH3CHeader(br.readLine()); //sڂ̃wbh
            
            /* Cell̉ */
            if(table == null){ return; }//e[u^ĂȂꍇ͏I
            ArrayList cellSet = table.getCellSet();
            if(cellSet != null ){
                cellSet.clear(); //e[u̗vf폜
                cellSet.add(0, null); //0Ԃ͎gpȂ̂ŏ
                //cellSet.add(0, new h3code.Cell(0)); //0Ԃ͎gpȂ̂ŏ (̕ł܂Ȃꍇ)
            }
            
            String line;
            while((line = br.readLine()) != null){
                System.out.println("This line length = "+line.length());//debug
                if(line.length() == 0){continue;} //s̍s
                StringBuffer sb = new StringBuffer(line);
                if(line.length() >0 && sb.charAt(0)=='%'){ continue; }  //Rg

                System.out.println("This line is "+line);//debug
                
                //C
                System.out.println("Analyze a cell line.");//debug
                analyzeCellLine(line); //s
                constructCell(table); //s͂ʂZ\zEo^
            }
        }catch(Exception e){
            System.out.println("load error = "+e.toString());
        }
        System.out.println("This File was loaded.");//debug
    }
    
    /* HTCt@C̃wb_[ */
    /**
     * Analyzes the head part of H3-Code
     * @param line  a line of H3code.
     * @return true: if <code>line</code> is correct. 
     */
    public boolean analyzeH3CHeader(String line){
        header = new Header();//wbhCX^X쐬
        StringTokenizer st = new StringTokenizer(line, " ");
        
        for(int state=1; st.hasMoreTokens(); state++){
            switch(state){
                case 1: //H3-Code Version
                    try{ setH3CVersion( Integer.parseInt(st.nextToken())); }
                    catch(Exception e){ return false; }
                    break;
                case 2: //Number of Rows
                    try{ setRow( Integer.parseInt(st.nextToken())); }
                    catch(Exception e){ return false; }
                    break;
                case 3: //Number of Columns
                    try{ setColumn( Integer.parseInt(st.nextToken())); }
                    catch(Exception e){ return false; }
                    break;
                default:
                    return false;
            }
        }
         return true;
    }
    
    
    /* Cell̈s */
    /**
     * analyzes a cell line. 
     * @param line  string
     * @return  true: if <code>line</code> is correct. 
     */
    public boolean analyzeCellLine(String line){
        cellLine = new CellLine(); //̃Zs̃CX^X쐬
        StringTokenizer st = new StringTokenizer(line, " ");
        
        for(int state=1; st.hasMoreTokens(); state++){
            switch(state){
                case 1: // node id (s1Ԗڂ̐l)
                    try{ setNodeID( Integer.parseInt(st.nextToken())); }
                    catch(Exception e){ return false; }
                    break;
                case 2: //cell̎
                    try{ setCellType( Integer.parseInt(st.nextToken())); }
                    catch(Exception e){ return false; }
                    break;
                case 3: //new_left
                    try{ setNWELeft( Integer.parseInt(st.nextToken())); }
                    catch(Exception e){ return false; }
                    break;
                case 4: //swe_left
                    try{ setSWELeft( Integer.parseInt(st.nextToken())); }
                    catch(Exception e){ return false; }
                    break;
                case 5: //nwe_right
                    try{ setNWERight( Integer.parseInt(st.nextToken())); }
                    catch(Exception e){ return false; }
                    break;
                case 6: //swe_right
                    try{ setSWERight( Integer.parseInt(st.nextToken())); }
                    catch(Exception e){ return false; }
                    break;
                
                    
                /* *********************************************************
                 * [ CӏiJnjE,Wt ] 060116
                 * ********************************************************* */
                case 7: //ewe_upper (C)
                    try{ setEWEUpper( Integer.parseInt(st.nextToken())); }
                    catch(Exception e){ return false; }
                    break;
                case 8: //wwe_upper (C)
                    try{ setWWEUpper( Integer.parseInt(st.nextToken())); }
                    catch(Exception e){ return false; }
                    break;
                case 9: //ewe_lower (C)
                    try{ setEWELower( Integer.parseInt(st.nextToken())); }
                    catch(Exception e){ return false; }
                    break;
                case 10: //wwe_lower (C)
                    try{ setWWELower( Integer.parseInt(st.nextToken())); }
                    catch(Exception e){ return false; }
                    break;
                /* *********************************************************
                 * [ CӏiIjE,Wt ]
                 * ********************************************************* */
                    
                    
                case 11: //north wall
                    try{ setNorthWall( Integer.parseInt(st.nextToken())); }
                    catch(Exception e){ return false; }
                    break;
                case 12: //south wall
                    try{ setSouthWall( Integer.parseInt(st.nextToken())); }
                    catch(Exception e){ return false; }
                    break;
                    
                
                /* *********************************************************
                 * [ CӏiJnjE,Wt ] 060116
                 * ********************************************************* */
                case 13: //east wall
                    try{ setEastWall( Integer.parseInt(st.nextToken())); }
                    catch(Exception e){ return false; }
                    break;
                case 14: //west wall
                    try{ setWestWall( Integer.parseInt(st.nextToken())); }
                    catch(Exception e){ return false; }
                    break;
                /* *********************************************************
                 * [ CӏiIjE,Wt ]
                 * ********************************************************* */
                    
                    
                case 15: //content id
                    try{ setContentID( Integer.parseInt(st.nextToken())); }
                    catch(Exception e){ return false; }
                    break;
                case 16: //content align
                    try{ setContentAlign( Integer.parseInt(st.nextToken())); }
                    catch(Exception e){ return false; }
                    break;
                case 17: //e_point (Ȃ)
                    break;
                default:
                    return false;    
            }
        }
        return true;
    }
    
    
    /* ȉ---eǂݍ񂾐l̉ */
    
    /* wb_[ */
    private void setH3CVersion(int version){
        if(header != null){ header.version = version; }
        System.out.println("H3-CODE Version"+version);
    }
    private void setRow(int row){
        if(header != null){ header.row = row; }
        System.out.println("ROW="+row);
    }
    private void setColumn(int column){
        if(header != null){ header.column = column; }
        System.out.println("Column="+column);
    }
    
    /* Z */
    private void setNodeID(int id){
        System.out.println("Cell ID = "+id);//debug
        if(cellLine != null){
            cellLine.nodeId = id;
            //System.out.println("Cell ID = "+id);//debug
        }
    }
    
    private void setCellType(int type){
        if(cellLine != null){ cellLine.type = type; }
    }
    
    private void setNWELeft(int value){
        if(cellLine != null){ cellLine.nweLeft = value; }
    }
    private void setSWELeft(int value){
        if(cellLine != null){ cellLine.sweLeft = value; }
    }
    private void setNWERight(int value){
        if(cellLine != null){ cellLine.nweRight = value; }
    }
    private void setSWERight(int value){
        if(cellLine != null){ cellLine.sweRight = value; }
    }
    private void setWWEUpper(int value){
        if(cellLine != null){ cellLine.wweUpper = value; }
    }
    private void setEWEUpper(int value){
        if(cellLine != null){ cellLine.eweUpper = value; }
    }
    private void setWWELower(int value){
        if(cellLine != null){ cellLine.wweLower = value; }
    }
    private void setEWELower(int value){
        if(cellLine != null){ cellLine.eweLower = value; }
    }
    private void setNorthWall(int value){
        if(cellLine != null){ cellLine.northWall = value; }
    }
    private void setSouthWall(int value){
        if(cellLine != null){ cellLine.southWall = value; }
    }
    private void setWestWall(int value){
        if(cellLine != null){ cellLine.westWall = value; }
    }
    private void setEastWall(int value){
        if(cellLine != null){ cellLine.eastWall = value; }
    }
    private void setContentID(int value){
        if(cellLine != null){ cellLine.contentId = value; }
    }
    private void setContentAlign(int value){
        if(cellLine != null){ cellLine.contentAlign = value; }
    }
    
    /* ݂̃Z\ɓo^ */
    /**
     * Adds a cell in <code>table</code>
     * @param table  a target table
     */
    private  void constructCell(org.hichart.h3code.Table table){
    	org.hichart.h3code.AbstractCell cell = null;
        if( cellLine.type == org.hichart.h3code.AbstractCell.PERIMETER_CELL){
        	org.hichart.h3code.PerimeterCell new_cell = new org.hichart.h3code.PerimeterCell(cellLine.nodeId);
            cell = new_cell;
        }else if(cellLine.type == org.hichart.h3code.AbstractCell.CELL){
        	org.hichart.h3code.Cell new_cell = new org.hichart.h3code.Cell(cellLine.nodeId);
//            if(cellLine.contentId != org.hichart.h3code.AbstractContentTypeTable.NONE){
//                new_cell.setContent(table.getContent(cellLine.contentId));
//            }else{
//                new_cell.setContent(null);
//            }
            new_cell.setContentAlign(cellLine.contentAlign);
            cell = new_cell;
        }else{
            return;
        }
        
        if(cell != null){
            /* 8ӂڑ */
            cell.connect(cellLine.nweLeft, org.hichart.h3code.AbstractCell.NWE_LEFT);
            cell.connect(cellLine.sweLeft, org.hichart.h3code.AbstractCell.SWE_LEFT);
            cell.connect(cellLine.nweRight, org.hichart.h3code.AbstractCell.NWE_RIGHT);
            cell.connect(cellLine.sweRight, org.hichart.h3code.AbstractCell.SWE_RIGHT);
            cell.connect(cellLine.wweUpper, org.hichart.h3code.AbstractCell.WWE_UPPER);
            cell.connect(cellLine.eweUpper, org.hichart.h3code.AbstractCell.EWE_UPPER);
            cell.connect(cellLine.wweLower, org.hichart.h3code.AbstractCell.WWE_LOWER);
            cell.connect(cellLine.eweLower, org.hichart.h3code.AbstractCell.EWE_LOWER);
        
            /* ǐݒ */
            cell.setAllWall(cellLine.northWall, cellLine.southWall, cellLine.westWall, cellLine.eastWall);
        
            /* o^ */
            table.addCell(cell.getID(), cell);
        }
    }
    
    /* 2005/02/11 */
    /* ǂ̒lӃZ̒lŏ */
    /* initWallValuesOfCells */
    /**
     * Initializes wall values of perimeter cells. 
     * @param table  a target cells
     */
    public void initWallValuesOfCells(org.hichart.h3code.Table table){
    	System.out.println("Wall initialization.");
    	
    	ArrayList cellSet = table.getCellSet(); //Z̏W𓾂D
    	
    	//EɈړȂǂ
    	AbstractCell row_p_cell = (AbstractCell)cellSet.get(table.getHead().getConnectedCell(AbstractLinkedNode.NWE_RIGHT)); //wbh̉E̎ӃZ𓾂
    	for(;row_p_cell!=null;){
    		
    		//EastWall̏
    		AbstractCell row_t_east_cell = (AbstractCell)cellSet.get(row_p_cell.getConnectedCell(AbstractLinkedNode.EWE_LOWER));
    		boolean repeat = row_t_east_cell instanceof Cell;
    		for( ;repeat; ){
    			row_t_east_cell.setWallValue(row_p_cell.getWallValue(AbstractCell.EAST),AbstractCell.EAST);
    			row_t_east_cell = (AbstractCell)cellSet.get(row_t_east_cell.getConnectedCell(AbstractLinkedNode.EWE_LOWER));
    			repeat = row_t_east_cell instanceof Cell;
    		}
    		
    		//WestWall̏
    		AbstractCell row_t_west_cell = (AbstractCell)cellSet.get(row_p_cell.getConnectedCell(AbstractLinkedNode.WWE_LOWER));
    		repeat = row_t_west_cell instanceof Cell;
    		for( ;repeat; ){
    			row_t_west_cell.setWallValue(row_p_cell.getWallValue(AbstractCell.WEST),AbstractCell.WEST);
    			row_t_west_cell = (AbstractCell)cellSet.get(row_t_west_cell.getConnectedCell(AbstractLinkedNode.WWE_LOWER));
    			repeat = row_t_west_cell instanceof Cell;
    		}
    		
    		
    		
    		//ӃZEɈړ(E̗Ɉړ)
    		row_p_cell = (AbstractCell)cellSet.get(row_p_cell.getConnectedCell(AbstractLinkedNode.NWE_RIGHT));
    	}
    	
    	
    	AbstractCell col_p_cell = (AbstractCell)cellSet.get(table.getHead().getConnectedCell(AbstractLinkedNode.EWE_LOWER));
    	for(;col_p_cell!=null;){
    		
    		//NorthWall̏
    		AbstractCell col_t_north_cell = (AbstractCell)cellSet.get(col_p_cell.getConnectedCell(AbstractLinkedNode.NWE_RIGHT));
    		boolean repeat = col_t_north_cell instanceof Cell;
    		for( ;repeat; ){
    			col_t_north_cell.setWallValue(col_p_cell.getWallValue(AbstractCell.NORTH),AbstractCell.NORTH);
    			col_t_north_cell = (AbstractCell)cellSet.get(col_t_north_cell.getConnectedCell(AbstractLinkedNode.NWE_RIGHT));
    			repeat = col_t_north_cell instanceof Cell;
    		}
    		
    		//SouthWall̏
    		AbstractCell col_t_south_cell = (AbstractCell)cellSet.get(col_p_cell.getConnectedCell(AbstractLinkedNode.SWE_RIGHT));
    		repeat = col_t_south_cell instanceof Cell;
    		for( ;repeat; ){
    			col_t_south_cell.setWallValue(col_p_cell.getWallValue(AbstractCell.SOUTH),AbstractCell.SOUTH);
    			col_t_south_cell = (AbstractCell)cellSet.get(col_t_south_cell.getConnectedCell(AbstractLinkedNode.SWE_RIGHT));
    			repeat = col_t_south_cell instanceof Cell;
    		}
    		
    		//ӃZɈړ(̍sɈړ)
    		col_p_cell = (AbstractCell)cellSet.get(col_p_cell.getConnectedCell(AbstractLinkedNode.EWE_LOWER));
    	}
    	
    }
    
    /* 2005/02/11 */
    /* ǂ̒lT */
    /* gp */
    /*
    private WallValue findWallValue(Cell cell, int position, Table table){
    	ArrayList cellSet = table.getCellSet();
    	org.hichart.h3code.AbstractCell tcell = cell;
    	
    	while(tcell.getWallValue(position) == null && tcell !=null){
    		tcell = (org.hichart.h3code.AbstractCell)cellSet.get(tcell.getConnectedCell(position));
    	}
    	
    	return tcell.getWallValue(position);
    }
    */
    
}
